﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using com.unity.mgobe;
using com.unity.mgobe.src.Util;
using System;

public class Client : MonoBehaviour
{

    PlayerInfo myPlayerInfo;

    List<Action> actionList = new List<Action>();
    static object onFrameLock = new object(); //lock


    //UI
    public LoginPanel loginPanel;
    public RoomPanel roomPanel;
    public CreateRoomPanel createRoomPanel;
    public RoomInPanel roomInPanel;


    //
    public PlayerInfoPara playerInfoPara {
        get => new PlayerInfoPara {
            Name = myPlayerInfo.Name,
            CustomPlayerStatus = myPlayerInfo.CustomPlayerStatus,
            CustomProfile = myPlayerInfo.CustomProfile
        };
    }

    // Start is called before the first frame update
    private void Start()
    {
        GenerateRandomOpenId();
        InitSDK();
    }

    private void Update() {
        
        if(actionList.Count != 0) {
            lock (onFrameLock) {
                foreach(var action in actionList) {
                    action?.Invoke();
                }
                actionList.Clear();
            }
        }
    }

    private void AddAction(Action ac) {
        lock (onFrameLock) {
            actionList.Add(ac);
        }
    }

    public void InitSDK() 
    {
        Global.GameId = "obg-9tog9kc2";
        Global.OpenId = loginPanel.myOpenId.text;
        Global.SecretKey = "2ceedb35a335f5db3f921b4fb9602a06db3c6a5a";
        Global.Server = "9tog9kc2.wxlagame.com";

        GameInfoPara gameInfo = new GameInfoPara {
            GameId = Global.GameId,
            OpenId = Global.OpenId,
            SecretKey = Global.SecretKey    
        };
        ConfigPara config = new ConfigPara {
            Url = Global.Server,
            ReconnectMaxTimes = 5,
            ReconnectInterval = 1000,
            ResendInterval = 1000,
            ResendTimeout = 10000
        };

        //初始化监听器Listener
        Listener.Init(gameInfo, config, (ResponseEvent eve) => {
            if(eve.Code == ErrCode.EcOk) {
                Debug.Log("初始化 Listener 成功");
                //创建房间实例
                Global.Room = new Room(null);
                Global.Room.GetRoomDetail((ResponseEvent e) => {
                    if(e.Code != 0 && e.Code != 20011) {
                        Debug.LogError("初始化失败");
                        return;
                    }
                      
                    if (e.Code == 0) {
                        var rsp = (GetRoomByRoomIdRsp)e.Data;
                        //设置房间信息到room实例
                        Global.Room.InitRoom(rsp.RoomInfo);
                        Debug.Log($"玩家已在房间内：{rsp.RoomInfo.Name}");
                        return;
                    }
                    if(e.Code == 20011) {
                        Debug.Log("玩家不在房间内");
                        return;
                    }
                });

                Listener.Add(Global.Room);
                loginPanel.loginBtn.interactable = true;
            } else {
                //初始化失败
                Debug.LogError("初始化 Listerner 失败");
            }

            //初始化广播回调事件

        });

        
    }

    void initBoardCast() {



        //设置房间改编广播回调
        Global.Room.OnChangeRoom = eve => {

        };

        //设置匹配房间成功广播回调
        Room.OnMatch = eve => {
            Debug.Log("正在匹配!");
        };

        //设置取消匹配房间广播回调
        Room.OnCancelMatch = eve => {
            Debug.Log("取消匹配");
        };

        //设置玩家加入房间广播回调
        Global.Room.OnJoinRoom = eve => {
            Debug.Log("新玩家加入房间");
        };

        //设置玩家退出房间广播回调
        Global.Room.OnLeaveRoom = eve => {
            Debug.Log("玩家推出房间");
        };

    }

    //登录
    public void Login() {
        loginPanel.loginBtn.interactable = false;

        string playerId = GamePlayerInfo.GetInfo().Id;
        loginPanel.myPlayerId.text = playerId;

        myPlayerInfo = new PlayerInfo();
        myPlayerInfo.Id = playerId;
        myPlayerInfo.Name = loginPanel.myPlayerId.text;
        myPlayerInfo.TeamId = "0";
        myPlayerInfo.CustomPlayerStatus = 0;
        myPlayerInfo.CustomProfile = "";

        Debug.Log(myPlayerInfo.Id);

        roomPanel.gameObject.SetActive(true);
        loginPanel.gameObject.SetActive(false);
        loginPanel.loginBtn.interactable = true;

    }

    //创建房间面板
    public void CreateRoomPanel() {
        createRoomPanel.canCreate = true;
        createRoomPanel.gameObject.SetActive(true);
    }

    //创建房间
    public void CreateRoom() {
        CreateRoomPara para = new CreateRoomPara {
            RoomName = createRoomPanel.getName(),
            MaxPlayers = createRoomPanel.getPlayerNum(),
            RoomType = "Test",
            CustomProperties = "0",
            IsPrivate = false,
            PlayerInfo = this.playerInfoPara,
        };

        Global.Room.CreateRoom(para, eve => {
            if(eve.Code == 0) {
                //更新房间列表
                Debug.Log("创建房间成功");
                AddAction(() => { OnRoomPanel(); });
            } else {
                Debug.Log($"create room faild: {eve.Code}");
                AddAction(() => { HideRoomPanel(); });
            }
        });
    }

    public void OnRoomPanel() {
        roomInPanel.gameObject.SetActive(true);
        createRoomPanel.canCreate = false;
        createRoomPanel.gameObject.SetActive(false);
    }

    public void HideRoomPanel() {
        createRoomPanel.canCreate = false;
        createRoomPanel.gameObject.SetActive(false);
    }

    //加入房间
    public void JoinRoom() {

    }

    public void MatchRoom() {

    }

    //退出房间
    public void LeaveRoom() {

    }

    public void GenerateRandomOpenId() {
        loginPanel.myOpenId.text =  $"Player{UnityEngine.Random.Range(1, 100)}";
    }

 
}
